﻿using LibOfdCreatorParam;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WpfInvoiceClient
{
    /// <summary>
    /// 生成发票数据，用于发送到服务端
    /// </summary>
    class InvoiceCreator
    {
        string TemplateGuid = "42967140-f282-4abc-9d64-53c72dad1657";

        int FirstPage_MaxGoodsInfoCount = 8;
        int GoodsInfoPage_MaxGoodsInfoCount = 26;

        public WebClientInputParam Creat(int goodsInfoCount)
        {
            WebClientInputParam clientParam = new WebClientInputParam();
            clientParam.TemplateGuid = TemplateGuid;
            clientParam.ClientData = "客户端发送的tag数据";

            clientParam.PageGroup = new List<WebClientPage>();

            //计算需要的发票页数
            int pageCount = 0;
            if (goodsInfoCount <= FirstPage_MaxGoodsInfoCount)
            {
                pageCount = 1;
            }
            else
            {
                pageCount = goodsInfoCount / GoodsInfoPage_MaxGoodsInfoCount;
                if (goodsInfoCount % GoodsInfoPage_MaxGoodsInfoCount != 0)
                {
                    pageCount++;
                }
                //加上首页
                pageCount++;
            }

            //生成发票首页
            if (pageCount == 1)
            {
                WebClientPage page = CreateFirstPage(goodsInfoCount);
                clientParam.PageGroup.Add(page);
            }
            else
            {
                //首页没有货物清单
                WebClientPage page = CreateFirstPage(0);
                clientParam.PageGroup.Add(page);

                //货物清单
                for (int pageIndex = 1; pageIndex < pageCount; pageIndex++)
                {
                    int goodInfoStart = (pageIndex - 1) * GoodsInfoPage_MaxGoodsInfoCount;
                    int goodInfoCount = GoodsInfoPage_MaxGoodsInfoCount;
                    //最后一页
                    bool isLastPage = (pageIndex == pageCount - 1);
                    if (isLastPage)
                    {
                        goodInfoCount = goodsInfoCount % GoodsInfoPage_MaxGoodsInfoCount;
                        if (goodInfoCount == 0)
                            goodInfoCount = GoodsInfoPage_MaxGoodsInfoCount;
                    }

                    page = CreateGoodInfoPage(pageCount - 1, pageIndex,
                        goodInfoStart, goodInfoCount, isLastPage);

                    clientParam.PageGroup.Add(page);
                }
            }

            return clientParam;
        }

        string _qrFilePath = "./发票二维码.png";
        /// <summary>
        /// 
        /// </summary>
        /// <param name="goodsInfoCount">包含货物清单的个数</param>
        /// <returns></returns>
        private WebClientPage CreateFirstPage(int goodsInfoCount)
        {
            WebClientPage page = new WebClientPage();
            page.TemplatePageIndex = 0;//对于模版文件oft的第一页

            if(File.Exists(_qrFilePath))
            {
                string imageData = Convert.ToBase64String(File.ReadAllBytes(_qrFilePath));
                WebClientImageItem image = new WebClientImageItem();
                image.Guid = "*40";
                image.DataValue = imageData;
                page.ListPageImage.Add(image);
            }

            //发票抬头
            WebClientTextItem item = new WebClientTextItem()
            { Guid = "*42", DataValue = "北京市电子发票OFD专票" };
            page.ListPageText.Add(item);

            //机器编号
            item = new WebClientTextItem()
            { Guid = "*134", DataValue = "435230623" };
            page.ListPageText.Add(item);

            /////**************发票右上方区域***************////

            //发票代码
            item = new WebClientTextItem()
            { Guid = "*120", DataValue = "1320465076325" };
            page.ListPageText.Add(item);

            //发票号码
            item = new WebClientTextItem()
            { Guid = "*122", DataValue = "546036346" };
            page.ListPageText.Add(item);

            //开票日期
            item = new WebClientTextItem()
            { Guid = "*124", DataValue = "2020年05月26日" };
            page.ListPageText.Add(item);

            //校验码
            item = new WebClientTextItem()
            { Guid = "*126", DataValue = "39506-01229036346736" };
            page.ListPageText.Add(item);


            /////**************购买方信息***************////

            //购买方名称
            item = new WebClientTextItem()
            { Guid = "*110", DataValue = "航天股份科技公司" };
            page.ListPageText.Add(item);

            //购买方-纳税人识别号
            item = new WebClientTextItem()
            { Guid = "*112", DataValue = "6042036037098M" };
            page.ListPageText.Add(item);

            //购买方-地址电话
            item = new WebClientTextItem()
            { Guid = "*114", DataValue = "北京市西城区188号楼14层18层 010-562325323" };
            page.ListPageText.Add(item);

            //购买方-开户行及账号
            item = new WebClientTextItem()
            { Guid = "*116", DataValue = "中国工商银行北京大兴桥支行3406530583636" };
            page.ListPageText.Add(item);

            //密码区
            item = new WebClientTextItem()
            { Guid = "*118", DataValue = "0011--9<+4-+/8+>-0<1<>157731\r\n<+/66/>8*//1+>75-36+99600178\r\n99*160///7-25/*<8>34+02>/-24\r\n892577>8548+690161611992<795" };
            page.ListPageText.Add(item);

            /////**************销售方信息***************////

            //销售方-名称
            item = new WebClientTextItem()
            { Guid = "*136", DataValue = "货物出口总公司" };
            page.ListPageText.Add(item);

            //销售方-纳税人识别号
            item = new WebClientTextItem()
            { Guid = "*138", DataValue = "074034032572312" };
            page.ListPageText.Add(item);

            //销售方-地址电话
            item = new WebClientTextItem()
            { Guid = "*140", DataValue = "重庆市哪个区某条路123号 电话432985592" };
            page.ListPageText.Add(item);

            //销售方-开户行及账号
            item = new WebClientTextItem()
            { Guid = "*142", DataValue = "重庆市中国银行市北支行" };
            page.ListPageText.Add(item);

            //备注
            item = new WebClientTextItem()
            { Guid = "*144", DataValue = "备注：这次开的是OFD专用票" };
            page.ListPageText.Add(item);

            //总计-金额
            item = new WebClientTextItem()
            { Guid = "*251", DataValue = "¥34.53" };
            page.ListPageText.Add(item);

            //总计-税额
            item = new WebClientTextItem()
            { Guid = "*253", DataValue = "¥976.3" };
            page.ListPageText.Add(item);

            //价税合计大写
            item = new WebClientTextItem()
            { Guid = "*86", DataValue = "壹万元零陆仟捌分整" };
            page.ListPageText.Add(item);

            //价税合计小写
            item = new WebClientTextItem()
            { Guid = "*90", DataValue = "¥186234.30" };
            page.ListPageText.Add(item);

            /////**********发票尾部*************///

            //收款人
            item = new WebClientTextItem()
            { Guid = "*146", DataValue = "李收款" };
            page.ListPageText.Add(item);

            //复核
            item = new WebClientTextItem()
            { Guid = "*148", DataValue = "刘复核" };
            page.ListPageText.Add(item);

            //开票人
            item = new WebClientTextItem()
            { Guid = "*150", DataValue = "张开票" };
            page.ListPageText.Add(item);

            //////////**********货物清单*************///////////
            if (goodsInfoCount == 0)
            {
                //首页无货物清单
                item = new WebClientTextItem()
                { Guid = "*128", DataValue = "（详见销货清单）" };
                page.ListPageText.Add(item);
            }
            else
            {
                for (int i = 0; i < goodsInfoCount; i++)
                {
                    List<WebClientTextItem> textOfGoodsInfo = GetGoodsInfo(true, i);

                    //需要指示，在第几行上显示 
                    if (i != 0)
                    {
                        textOfGoodsInfo.ForEach(o => o.ItemIndex = i);
                    }

                    page.ListPageText.AddRange(textOfGoodsInfo);
                }
            }
            return page;
        }

        private WebClientPage CreateGoodInfoPage(int totalPageCount, int pageIndex,
            int goodInfoStart, int goodInfoCount, bool isLastPage)
        {
            WebClientPage page = new WebClientPage();
            page.TemplatePageIndex = 1;//对于模版文件oft的第2页

            /////**************货物清单上方区域***************////

            //购买方名称
            WebClientTextItem item = new WebClientTextItem()
            { Guid = "*189", DataValue = "航天信息货物清单页" };
            page.ListPageText.Add(item);

            //销售方名称
            item = new WebClientTextItem()
            { Guid = "*191", DataValue = "ukey1000套" };
            page.ListPageText.Add(item);

            //发票代码
            item = new WebClientTextItem()
            { Guid = "*193", DataValue = "4723523523" };
            page.ListPageText.Add(item);

            //发票号码
            item = new WebClientTextItem()
            { Guid = "*195", DataValue = "9674574" };
            page.ListPageText.Add(item);

            //总页数
            item = new WebClientTextItem()
            { Guid = "*197", DataValue = totalPageCount.ToString() };
            page.ListPageText.Add(item);

            //第几页
            item = new WebClientTextItem()
            { Guid = "*199", DataValue = pageIndex.ToString() };
            page.ListPageText.Add(item);

            //具体货物清单详情
            for (int i = 0; i < goodInfoCount; i++)
            {
                int goodInfoIndex = i + goodInfoStart;
                List<WebClientTextItem> textOfGoodsInfo = GetGoodsInfo(false, goodInfoIndex);

                //需要指示，在第几行上显示 
                if (i != 0)
                {
                    textOfGoodsInfo.ForEach(o => o.ItemIndex = i);
                }

                page.ListPageText.AddRange(textOfGoodsInfo);
            }

            //小计、折扣、总计
            List<WebClientTextItem> textItemSummary = CreateGoodInfoSummary(goodInfoStart,
                    goodInfoCount, isLastPage);
            page.ListPageText.AddRange(textItemSummary);

            //备注
            item = new WebClientTextItem()
            { Guid = "*257", DataValue = "粮食储备！！！" };
            page.ListPageText.Add(item);

            //开票日期
            item = new WebClientTextItem()
            { Guid = "*261", DataValue = "2020年05月26日" };
            page.ListPageText.Add(item);

            return page;
        }

        //货物清单每行的高度 毫米
        float _goodInfoLineHeight = 6;

        List<WebClientTextItem> CreateGoodInfoSummary(int goodInfoStart,
            int goodInfoCount, bool isLastPage)
        {
            List<WebClientTextItem> textItems = new List<WebClientTextItem>();

            ////////**********小计**********//////
            //标题
            WebClientTextItem item = new WebClientTextItem()
            { Guid = "*241", DataValue = "小计" };
            textItems.Add(item);

            //小计-金额
            item = new WebClientTextItem()
            { Guid = "*243", DataValue = "¥352.09" };
            textItems.Add(item);

            //小计-税率
            item = new WebClientTextItem()
            { Guid = "*245", DataValue = "¥349.19" };
            textItems.Add(item);

            ////////**********折扣**********//////
            //折扣-标题
            item = new WebClientTextItem()
            { Guid = "*239", DataValue = "折扣" };
            textItems.Add(item);

            //折扣-金额
            item = new WebClientTextItem()
            { Guid = "*247", DataValue = "¥843.3" };
            textItems.Add(item);

            //折扣-税率
            item = new WebClientTextItem()
            { Guid = "*249", DataValue = "¥21.3" };
            textItems.Add(item);

            if (isLastPage)
            {
                //尾页才有总计

                //总计-标题
                item = new WebClientTextItem()
                { Guid = "*237", DataValue = "总计" };
                textItems.Add(item);

                //总计-金额
                item = new WebClientTextItem()
                { Guid = "*251", DataValue = "¥34.53" };
                textItems.Add(item);

                //总计-税额
                item = new WebClientTextItem()
                { Guid = "*253", DataValue = "¥976.3" };
                textItems.Add(item);
            }
            else
            {
                //续页没有总计，所以小计、折扣位置向下移动一行
                textItems.ForEach(o => o.OffsetY = _goodInfoLineHeight);
            }
            return textItems;
        }

        private List<WebClientTextItem> GetGoodsInfo(bool isFristPage, int goodsIndex)
        {
            List<WebClientTextItem> textItems = new List<WebClientTextItem>();

            WebClientTextItem item;

            //不是发票首页 有”序号“栏
            if (!isFristPage)
            {
                //序号
                item = new WebClientTextItem()
                { Guid = "*219", DataValue = (goodsIndex + 1).ToString() };
                textItems.Add(item);
            }

            //项目名称
            item = new WebClientTextItem()
            { Guid = "*221", DataValue = "大米 小麦" };
            textItems.Add(item);

            //规格型号
            item = new WebClientTextItem()
            { Guid = "*223", DataValue = "优级" };
            textItems.Add(item);

            //单位
            item = new WebClientTextItem()
            { Guid = "*225", DataValue = "千克" };
            textItems.Add(item);

            //数量
            item = new WebClientTextItem()
            { Guid = "*227", DataValue = "789" };
            textItems.Add(item);

            //单价
            item = new WebClientTextItem()
            { Guid = "*229", DataValue = "58.97" };
            textItems.Add(item);

            //金额
            item = new WebClientTextItem()
            { Guid = "*231", DataValue = "321.7" };
            textItems.Add(item);

            //税率
            item = new WebClientTextItem()
            { Guid = "*233", DataValue = "95.6%" };
            textItems.Add(item);

            //税额
            item = new WebClientTextItem()
            { Guid = "*235", DataValue = "832" };
            textItems.Add(item);

            return textItems;
        }
    }
}
