﻿using LibOfdCreatorParam;
using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Threading;

namespace WpfInvoiceClient
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            _wndTimer = new DispatcherTimer();
            _wndTimer.Interval = TimeSpan.FromSeconds(1);
            _wndTimer.Tick += dtimer_Tick;
            _wndTimer.Start();

            List<int> items = new List<int>() { 1, 10, 100, 1000, 10000 };
            comboOfdCreateCount.ItemsSource = items;
            comboOfdCreateCount.SelectedIndex = 0;
        }

        long _ofdFileCount;
        long _ofdFileCountPre;
        private void dtimer_Tick(object sender, EventArgs e)
        {
            long speed = _ofdFileCount - _ofdFileCountPre;
            _ofdFileCountPre = _ofdFileCount;

            txtOfdFileCount.Text = _ofdFileCount.ToString();
            txtOfdFileSpeed.Text = speed.ToString();
        }

        EN_CreateType GetCreateType()
        {
            if (radioFapiao.IsChecked == true)
            {
                return EN_CreateType.发票;
            }

            if (radioJiehunzheng.IsChecked == true)
            {
                return EN_CreateType.结婚证;
            }
            return EN_CreateType.发票;
        }


        volatile int _curOfdCreateTask = 0; //当前并发的线程
        volatile int _ofdCreateCountLeft;//还可以生成多少个ofd文件
        volatile int _callThreadCount = 10;
        private void txtCallThreadCount_TextChanged(object sender, TextChangedEventArgs e)
        {
            try
            {
                _callThreadCount = int.Parse(txtCallThreadCount.Text.Trim());
                if (_callThreadCount < 0)
                {
                    _callThreadCount = 1;
                }
                if (_callThreadCount > 100)
                {
                    _callThreadCount = 100;
                }
            }
            catch (Exception)
            {
                txtCallThreadCount.Text = "10";
            }
        }

        DateTime _startTimeBatchCreate = DateTime.Now;
        private async void btnCreateOfdBatch_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                txtBatchCreateSpan.Text = string.Empty;
                string url = txtHttpUrl.Text.Trim();
                int totalCreateCount = int.Parse(comboOfdCreateCount.Text.Trim());
                _ofdCreateCountLeft = totalCreateCount;
                _curOfdCreateTask = 0;

                EN_CreateType createType = GetCreateType();
                _startTimeBatchCreate = DateTime.Now;
                await Task.Run(() =>
                {
                    while (true)
                    {
                        if (_ofdCreateCountLeft <= 0)
                        {
                            break;
                        }

                        //客户端同时有_nCallThreadCount个并发
                        if (_curOfdCreateTask >= _callThreadCount)
                        {
                            Thread.Sleep(1);
                            continue;
                        }

                        Interlocked.Decrement(ref _ofdCreateCountLeft);

                        try
                        {
                            CreateOfdFile(url, createType);
                        }
                        catch (Exception ex)
                        {
                            AddLog(ex.Message);
                            break;
                        }
                    }

                    //等生成结束
                    while (_curOfdCreateTask != 0)
                    {
                    }
                });

                double seconds = (DateTime.Now - _startTimeBatchCreate).TotalSeconds;
                double speed = totalCreateCount / seconds;
                txtBatchCreateSpan.Text = string.Format($"耗时: {seconds.ToString("f2")}秒,速度: {speed.ToString("f2")} 个/秒");
            }
            catch (Exception ex)
            {
                AddLog(ex.Message);
            }
        }


        int _googInfoCount = 8;//货物清单的个数
        private void CreateOfdFile(string url, EN_CreateType createType)
        {
            WebClientInputParam inputParam = null;
            if (createType == EN_CreateType.发票)
            {
                InvoiceCreator creator = new InvoiceCreator();
                inputParam = creator.Creat(_googInfoCount);
            }
            else if (createType == EN_CreateType.结婚证)
            {
                MarriageCertCreator creator = new MarriageCertCreator();
                inputParam = creator.Creat();
            }

            WebClientOutputParam ofdOutputParam = null;
            Interlocked.Increment(ref _curOfdCreateTask);

            Task.Run(() =>
            {
                try
                {
                    DateTime start = DateTime.Now;

                    OfdConvertorCall call = new OfdConvertorCall();
                    call.HttpServerUrl = url;
                    call.ConvertorCall(inputParam, out ofdOutputParam);

                    TimeSpan span = DateTime.Now - start;

                    DealOfdOutParam(ofdOutputParam, span);
                }
                catch (Exception ex)
                {
                    AddLog(ex.Message);
                }

                Interlocked.Decrement(ref _curOfdCreateTask);
                if (_curOfdCreateTask < 0)
                    _curOfdCreateTask = 0;
            });
        }

        int _ofdFileIndex;
        private void DealOfdOutParam(WebClientOutputParam ofdOutputParam, TimeSpan span)
        {
            if (ofdOutputParam.OK)
            {
                string fileName = string.Empty;

                Interlocked.Increment(ref _ofdFileCount);
                Interlocked.Increment(ref _ofdFileIndex);
                int fileIndex = _ofdFileIndex;
                if (_isShowLog)
                {
                    fileName = string.Format(".\\ofd\\{0}_{1}.ofd",
                        DateTime.Now.ToString("yyyyMMddHHmmss"), fileIndex);
                    AddLog(string.Format("生成成功 {0};耗时:{1}毫秒", fileName, span.TotalMilliseconds));
                }

                if (_isSaveOfdFile)
                {
                    Directory.CreateDirectory(".\\ofd");
                    byte[] fileData = Convert.FromBase64String(ofdOutputParam.OfdFileDataBase64);
                    File.WriteAllBytes(fileName, fileData);
                }
            }
            else
            {
                if (_isShowLog)
                {
                    AddLog(string.Format("生成失败;耗时:{0}毫秒.{1}",
                    span.TotalMilliseconds, ofdOutputParam.Message));
                }
            }
        }


        bool _isSaveOfdFile = true;
        bool _isShowLog = true;
        private DispatcherTimer _wndTimer;

        void AddLog(string message)
        {
            if (!_isShowLog)
                return;

            this.Dispatcher.Invoke(() => AddLogInner(message));
        }

        void AddLogInner(string message)
        {
            LogMessage logMessage = new LogMessage() { Str = new string[3] };
            logMessage.Str[0] = (ListLogInfo.Items.Count + 1).ToString();
            logMessage.Str[1] = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            logMessage.Str[2] = message;
            ListLogInfo.Items.Add(logMessage);
        }

        private void menuItemClearAll_Click(object sender, RoutedEventArgs e)
        {
            ListLogInfo.Items.Clear();
        }

        private void checkSaveOfd_Click(object sender, RoutedEventArgs e)
        {
            _isSaveOfdFile = checkSaveOfd.IsChecked == true;
        }

        private void checkShowLog_Click(object sender, RoutedEventArgs e)
        {
            _isShowLog = checkShowLog.IsChecked == true;
        }

        private void txtGoogInfoCount_TextChanged(object sender, TextChangedEventArgs e)
        {
            try
            {
                _googInfoCount = int.Parse(txtGoogInfoCount.Text.Trim());
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                txtGoogInfoCount.Text = _googInfoCount.ToString();
            }
        }

        private void btnOpenSaveDir_Click(object sender, RoutedEventArgs e)
        {
            System.Diagnostics.Process.Start("explorer.exe", ".\\ofd");
        }
    }

    public class LogMessage
    {
        public string[] Str { get; set; }
    }

    public enum EN_CreateType
    {
        发票,
        结婚证
    }
}
